'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _versions = require('../versions');

var _versions2 = _interopRequireDefault(_versions);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _bs = require('bs58');

var _bs2 = _interopRequireDefault(_bs);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _elliptic = require('elliptic');

var _elliptic2 = _interopRequireDefault(_elliptic);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const HARDENED_KEY_OFFSET = 0x80000000;
const secp256k1 = new _elliptic2.default.ec('secp256k1');
class HDKey {
    constructor({ privateKey, publicKey, chainCode, index, depth, parentFingerprint, version }) {
        if (!privateKey && !publicKey) {
            throw new Error('either private key or public key must be provided');
        }
        if (privateKey) {
            this._privateKey = privateKey;
            const ecdh = _crypto2.default.createECDH('secp256k1');
            if (ecdh.curve && ecdh.curve.keyFromPrivate) {
                // ECDH is not native, fallback to pure-JS elliptic lib
                this._publicKey = Buffer.from(secp256k1.keyFromPrivate(privateKey).getPublic(true, 'hex'), 'hex');
            } else {
                ecdh.setPrivateKey(privateKey);
                this._publicKey = Buffer.from(ecdh.getPublicKey('latin1', 'compressed'), 'latin1');
            }
        } else if (publicKey) {
            this._publicKey = publicKey;
        }
        this._chainCode = chainCode;
        this._depth = depth || 0;
        this._index = index || 0;
        this._parentFingerprint = parentFingerprint;
        this._keyIdentifier = hash160(this._publicKey);
        this._version = version || _versions2.default.bitcoinMain;
    }
    static parseMasterSeed(seed, version) {
        const i = hmacSha512('Bitcoin seed', seed);
        const iL = i.slice(0, 32);
        const iR = i.slice(32);
        return new HDKey({ privateKey: iL, chainCode: iR, version });
    }
    static parseExtendedKey(key, version = _versions2.default.bitcoinMain) {
        // version_bytes[4] || depth[1] || parent_fingerprint[4] || index[4] || chain_code[32] || key_data[33] || checksum[4]
        const decoded = Buffer.from(_bs2.default.decode(key));
        if (decoded.length > 112) {
            throw new Error('invalid extended key');
        }
        const checksum = decoded.slice(-4);
        const buf = decoded.slice(0, -4);
        if (!sha256(sha256(buf)).slice(0, 4).equals(checksum)) {
            throw new Error('invalid checksum');
        }
        let o = 0;
        const versionRead = buf.readUInt32BE(o);
        o += 4;
        const depth = buf.readUInt8(o);
        o += 1;
        let parentFingerprint = buf.slice(o, o += 4);
        if (parentFingerprint.readUInt32BE(0) === 0) {
            parentFingerprint = undefined;
        }
        const index = buf.readUInt32BE(o);
        o += 4;
        const chainCode = buf.slice(o, o += 32);
        const keyData = buf.slice(o);
        const privateKey = keyData[0] === 0 ? keyData.slice(1) : undefined;
        const publicKey = keyData[0] !== 0 ? keyData : undefined;
        if (privateKey && versionRead !== version.bip32.private || publicKey && versionRead !== version.bip32.public) {
            throw new Error('invalid version bytes');
        }
        return new HDKey({
            privateKey,
            publicKey,
            chainCode,
            index,
            depth,
            parentFingerprint,
            version
        });
    }
    get privateKey() {
        return this._privateKey || null;
    }
    get publicKey() {
        return this._publicKey;
    }
    get chainCode() {
        return this._chainCode;
    }
    get depth() {
        return this._depth;
    }
    get parentFingerprint() {
        return this._parentFingerprint || null;
    }
    get index() {
        return this._index;
    }
    get keyIdentifier() {
        return this._keyIdentifier;
    }
    get fingerprint() {
        return this._keyIdentifier.slice(0, 4);
    }
    get version() {
        return this._version;
    }
    get extendedPrivateKey() {
        return this._privateKey ? this.serialize(this._version.bip32.private, this._privateKey) : null;
    }
    get extendedPublicKey() {
        return this.serialize(this._version.bip32.public, this._publicKey);
    }
    derive(chain) {
        const c = chain.toLowerCase();
        let childKey = this;
        c.split('/').forEach(path => {
            const p = path.trim();
            if (p === 'm' || p === "m'" || p === '') {
                return;
            }
            const index = Number.parseInt(p, 10);
            if (Number.isNaN(index)) {
                throw new Error('invalid child key derivation chain');
            }
            const hardened = p.slice(-1) === "'";
            childKey = childKey.deriveChildKey(index, hardened);
        });
        return childKey;
    }
    deriveChildKey(childIndex, hardened) {
        if (childIndex >= HARDENED_KEY_OFFSET) {
            throw new Error('invalid index');
        }
        if (!this.privateKey && !this.publicKey) {
            throw new Error('either private key or public key must be provided');
        }
        let index = childIndex;
        const data = Buffer.alloc(37);
        let o = 0;
        if (hardened) {
            if (!this.privateKey) {
                throw new Error('cannot derive a hardened child key from a public key');
            }
            // 0x00 || ser256(kpar) || ser32(i)
            // 0x00[1] || parent_private_key[32] || child_index[4]
            index += HARDENED_KEY_OFFSET;
            o += 1;
            o += this.privateKey.copy(data, o);
        } else {
            // serP(point(kpar)) || ser32(i)
            // compressed_parent_public_key[33] || child_index[4]
            o += this.publicKey.copy(data, o);
        }
        o += data.writeUInt32BE(index, o);
        const i = hmacSha512(this.chainCode, data);
        const iL = new _bn2.default(i.slice(0, 32));
        const iR = i.slice(32);
        // if parse256(IL) >= n, the resulting key is invalid; proceed with the next value for i
        if (iL.cmp(secp256k1.n) >= 0) {
            return this.deriveChildKey(childIndex + 1, hardened);
        }
        if (this.privateKey) {
            // ki is parse256(IL) + kpar (mod n)
            const childKey = iL.add(new _bn2.default(this.privateKey)).mod(secp256k1.n);
            // if ki = 0, the resulting key is invalid; proceed with the next value for i
            if (childKey.cmp(new _bn2.default(0)) === 0) {
                return this.deriveChildKey(childIndex + 1, hardened);
            }
            return new HDKey({
                depth: this.depth + 1,
                privateKey: childKey.toArrayLike(Buffer, 'be', 32),
                chainCode: iR,
                parentFingerprint: this.fingerprint,
                index,
                version: this.version
            });
        } else {
            // Ki is point(parse256(IL)) + Kpar = G * IL + Kpar
            const parentKey = secp256k1.keyFromPublic(this.publicKey).pub;
            const childKey = secp256k1.g.mul(iL).add(parentKey);
            // if Ki is the point at infinity, the resulting key is invalid; proceed with the next value for i
            if (childKey.isInfinity()) {
                return this.deriveChildKey(childIndex + 1, false);
            }
            const compressedChildKey = Buffer.from(childKey.encode(null, true));
            return new HDKey({
                depth: this.depth + 1,
                publicKey: compressedChildKey,
                chainCode: iR,
                parentFingerprint: this.fingerprint,
                index,
                version: this.version
            });
        }
    }
    serialize(version, key) {
        // version_bytes[4] || depth[1] || parent_fingerprint[4] || index[4] || chain_code[32] || key_data[33] || checksum[4]
        const buf = Buffer.alloc(78);
        let o = buf.writeUInt32BE(version, 0);
        o = buf.writeUInt8(this.depth, o);
        o += this.parentFingerprint ? this.parentFingerprint.copy(buf, o) : 4;
        o = buf.writeUInt32BE(this.index, o);
        o += this.chainCode.copy(buf, o);
        o += 33 - key.length;
        key.copy(buf, o);
        const checksum = sha256(sha256(buf)).slice(0, 4);
        return _bs2.default.encode(Buffer.concat([buf, checksum]));
    }
}
exports.default = HDKey;
function hmacSha512(key, data) {
    return _crypto2.default.createHmac('sha512', key).update(data).digest();
}
function sha256(data) {
    return _crypto2.default.createHash('sha256').update(data).digest();
}
function hash160(data) {
    const d = _crypto2.default.createHash('sha256').update(data).digest();
    return _crypto2.default.createHash('rmd160').update(d).digest();
}