'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _wordlist = require('./wordlist.en');

var _wordlist2 = _interopRequireDefault(_wordlist);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Mnemonic {
    constructor(entropy, words) {
        this._entropy = entropy;
        this._words = words;
    }
    static generate(entropy) {
        if (entropy.length % 4 !== 0) {
            return null;
        }
        const ent = entropy.length * 8;
        const cs = ent / 32;
        const bits = [].concat(...[...entropy].map(uint8ToBitArray));
        const shasum = _crypto2.default.createHash('sha256').update(entropy).digest();
        const checksum = [].concat(...[...shasum].map(uint8ToBitArray)).slice(0, cs);
        bits.push(...checksum);
        const words = [];
        for (let i = 0; i < bits.length / 11; i++) {
            const idx = elevenBitsToInt(bits.slice(i * 11, (i + 1) * 11));
            words.push(_wordlist2.default[idx]);
        }
        return new Mnemonic(entropy, words);
    }
    static parse(phrase) {
        const words = phrase.normalize('NFKD').split(' ');
        if (words.length % 3 !== 0) return null;
        const bitArrays = [];
        for (let i = 0; i < words.length; i++) {
            const word = words[i];
            const idx = _wordlist2.default.indexOf(word);
            if (idx === -1) return null;
            bitArrays.push(uint11ToBitArray(idx));
        }
        const bits = [].concat(...bitArrays);
        const cs = bits.length / 33;
        if (cs !== Math.floor(cs)) return null;
        const checksum = bits.slice(-cs);
        bits.splice(-cs, cs);
        const entropy = [];
        for (let i = 0; i < bits.length / 8; i++) {
            entropy.push(eightBitsToInt(bits.slice(i * 8, (i + 1) * 8)));
        }
        const entropyBuf = Buffer.from(entropy);
        const shasum = _crypto2.default.createHash('sha256').update(entropyBuf).digest();
        const checksumFromSha = [].concat(...[...shasum].map(uint8ToBitArray)).slice(0, cs);
        if (!arraysEqual(checksumFromSha, checksum)) return null;
        return new Mnemonic(entropyBuf, words);
    }
    get entropy() {
        return this._entropy;
    }
    get words() {
        return this._words;
    }
    get phrase() {
        if (!this._phrase) {
            this._phrase = this._words.join(' ');
        }
        return this._phrase;
    }
    toSeed(passphrase = '') {
        const salt = `mnemonic${passphrase}`;
        return Mnemonic.pbkdf2Sync(this.phrase.normalize('NFKD'), salt.normalize('NFKD'), 2048, 64, 'sha512');
    }
    toSeedAsync(passphrase = '') {
        const salt = `mnemonic${passphrase}`;
        return new Promise((resolve, reject) => {
            Mnemonic.pbkdf2(this.phrase.normalize('NFKD'), salt.normalize('NFKD'), 2048, 64, 'sha512', (err, key) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(key);
            });
        });
    }
}
exports.default = Mnemonic;
Mnemonic.pbkdf2Sync = _crypto2.default.pbkdf2Sync;
Mnemonic.pbkdf2 = _crypto2.default.pbkdf2;
function uint11ToBitArray(n) {
    return [Math.min(n & 1024, 1), Math.min(n & 512, 1), Math.min(n & 256, 1), Math.min(n & 128, 1), Math.min(n & 64, 1), Math.min(n & 32, 1), Math.min(n & 16, 1), Math.min(n & 8, 1), Math.min(n & 4, 1), Math.min(n & 2, 1), Math.min(n & 1, 1)];
}
function uint8ToBitArray(n) {
    return [Math.min(n & 128, 1), Math.min(n & 64, 1), Math.min(n & 32, 1), Math.min(n & 16, 1), Math.min(n & 8, 1), Math.min(n & 4, 1), Math.min(n & 2, 1), Math.min(n & 1, 1)];
}
function elevenBitsToInt(bits) {
    return bits[0] * 1024 + bits[1] * 512 + bits[2] * 256 + bits[3] * 128 + bits[4] * 64 + bits[5] * 32 + bits[6] * 16 + bits[7] * 8 + bits[8] * 4 + bits[9] * 2 + bits[10];
}
function eightBitsToInt(bits) {
    return bits[0] * 128 + bits[1] * 64 + bits[2] * 32 + bits[3] * 16 + bits[4] * 8 + bits[5] * 4 + bits[6] * 2 + bits[7];
}
function arraysEqual(a, b) {
    if (a === b) return true;
    if (a.length !== b.length) return false;
    for (let i = 0; i < a.length; i++) {
        if (a[i] !== b[i]) return false;
    }
    return true;
}